#ifndef IOCTL_MAN_H
#define IOCTL_MAN_H

#define MIL1553UDx_DRIVER_MASKNAME "PCIe_1553UDx"

// Device type
#define PCIe_1553UD		1
#define PCIe_1553UD2	2
#define PCIe_1553UD4	4

// Channel number
#define ALLCH_UD	0
#define CH1_UD		1
#define CH2_UD		2
#define CH3_UD		3
#define CH4_UD		4

// Registers
#define DMA_DATA_BASE		0x1000
#define DMA_DATA_BASE_HI	0x1004
#define INTERRUPT_REG		0x100c
#define INTERRUPT_MASK_REG	0x1010

#define DMA_INDEX_1CH		0x1040
#define DMA_INDEX_2CH		0x1044
#define DMA_INDEX_3CH		0x1048
#define DMA_INDEX_4CH		0x104C

#define CTRL_REG_PCI_1CH	0x2000
#define CTRL_REG_PCI_2CH	0x4000
#define CTRL_REG_PCI_3CH	0x6000
#define CTRL_REG_PCI_4CH	0x8000

#define RTBM_CONF_REG_PCI_1CH	0x2004
#define RTBM_CONF_REG_PCI_2CH	0x4004
#define RTBM_CONF_REG_PCI_3CH	0x6004
#define RTBM_CONF_REG_PCI_4CH	0x8004

#define BC_CONF_REG_PCI_1CH	0x2008
#define BC_CONF_REG_PCI_2CH	0x4008
#define BC_CONF_REG_PCI_3CH	0x6008
#define BC_CONF_REG_PCI_4CH	0x8008

#define MIL_RT_INT_REG_1CH	0x2034
#define MIL_RT_INT_REG_2CH	0x4034
#define MIL_RT_INT_REG_3CH	0x6034
#define MIL_RT_INT_REG_4CH	0x8034

#define HW_STAT_REG2_1CH 	0x2084
#define HW_STAT_REG2_2CH 	0x4084
#define HW_STAT_REG2_3CH 	0x6084
#define HW_STAT_REG2_4CH 	0x8084

#define RT_RCV_REGn_1CH		0x2100
#define RT_RCV_REGn_2CH		0x4100
#define RT_RCV_REGn_3CH		0x6100
#define RT_RCV_REGn_4CH		0x8100

#define RT_TR_REGn_1CH		0x2180
#define RT_TR_REGn_2CH		0x4180
#define RT_TR_REGn_3CH		0x6180
#define RT_TR_REGn_4CH		0x8180

#define RT_DATA_BUF0n_1CH	0x2800
#define RT_DATA_BUF1n_1CH	0x3000
#define RT_DATA_BUF0n_2CH	0x4800
#define RT_DATA_BUF1n_2CH	0x5000
#define RT_DATA_BUF0n_3CH	0x6800
#define RT_DATA_BUF1n_3CH	0x7000
#define RT_DATA_BUF0n_4CH	0x8800
#define RT_DATA_BUF1n_4CH	0x9000

#define MIL_BC_DATA_RAM_1CH	0x10000
#define MIL_BC_DATA_RAM_2CH	0x20000
#define MIL_BC_DATA_RAM_3CH	0x30000
#define MIL_BC_DATA_RAM_4CH	0x40000

#define MIL_BC_INSTR_RAM_1CH	0x18000
#define MIL_BC_INSTR_RAM_2CH	0x28000
#define MIL_BC_INSTR_RAM_3CH	0x38000
#define MIL_BC_INSTR_RAM_4CH	0x48000

#define MIL_BC_OPERATION_RAM_1CH	0x1c000
#define MIL_BC_OPERATION_RAM_2CH	0x2c000
#define MIL_BC_OPERATION_RAM_3CH	0x3c000
#define MIL_BC_OPERATION_RAM_4CH	0x4c000

#define MAJO 'b'
// read/write regs
#define IOCTL_WR_REG_OFFSET _IOW(MAJO, 0, SADDR_DATA) //write to reg
#define IOCTL_RD_REG_OFFSET _IOWR(MAJO, 2, SADDR_DATA)	//write to reg
//!!! #define IOCTL_WR_REG _IOWR(MAJO, 1, SADDR_DATA) 		//DON'T USE, read from reg
//!!! #define IOCTL_RD_REG _IOWR(MAJO, 3, SADDR_DATA)		//DON'T USE, read from reg
// MODE
#define IOCTL_SWITCH_MODE _IOW(MAJO, 4, MODE_STR) // switch mode between BC, RT, BM, BM + ADDR
#define IOCTL_SET_BUS _IOW(MAJO, 33, BUS_SEL)
#define IOCTL_WORK_ENABLE _IOW(MAJO, 34, WORK_ENABLE)
#define IOCTL_SET_TR_PROG_MODE _IOW(MAJO, 35, SET_TR_MODE)
#define IOCTL_SET_TR_AUTO_MODE _IOW(MAJO, 36, SET_TR_MODE)
#define IOCTL_SET_TR_BUF_READY  _IOW(MAJO, 37, SET_TR_MODE_BUF)
// DMA
#define IOCTL_CLEAR_DMA _IOW(MAJO, 5, CH_NUM)       // reaset pointer DMA
#define IOCTL_RD_RAW_DMA _IOWR(MAJO, 6, DMA_STR) // read DMA
#define IOCTL_GET_NBLOCK_RAW_DMA _IOWR(MAJO, 7, GET_NBLOCK) // get number of blocks in DMA
#define IOCTL_ENABLE_DMA _IO(MAJO, 8) 			//enable dma
#define IOCTL_DISABLE_DMA _IO(MAJO, 9) 			//disable dma
// IRQ
#define IOCTL_DISABLE_INTERRUPT _IOW(MAJO, 15, SINTHIOSA) // disable IRQ ERR
#define IOCTL_ENABLE_INTERRUPT _IOW(MAJO, 16, SINTHIOSA) // enable some IRQs
#define IOCTL_SUBADDRESS_IRQ_TRANSMIT  _IOW(MAJO, 38, SADDR_INT)
#define IOCTL_SUBADDRESS_IRQ_RECEIVE  _IOW(MAJO, 39, SADDR_INT)
// channel
//!!! #define IOCTL_ENABLE_CHANNEL _IO(MAJO, 17) 			//DON'T USE, enable channel
//!!! #define IOCTL_DISABLE_CHANNEL _IO(MAJO, 18) 			//DON'T USE, disable channel
#define IOCTL_SET_ADDRESS _IOW(MAJO, 19, AD_STR)    //set address
#define IOCTL_SET_TIMER_RTBM_CONF_REG_PCI _IOW(MAJO, 20, TT) //set timer and timeout
#define IOCTL_SET_TIMER_BC_CONF_REG_PCI _IOW(MAJO, 32, TT) //set timer and timeout
// paddr and buf
#define IOCTL_ENABLE_SUBADDR _IOW(MAJO, 21, SADDR_DATA_PA) //enable paddr
#define IOCTL_DISABLE_SUBADDR _IOW(MAJO, 22, SADDR_DATA_PA) //disable paddr
//!!! #define IOCTL_WR_REG_PA_RT _IOWR(MAJO, 23, SADDR_DATA_PA) 		//DON'T USE, write to paddr RT
//!!! #define IOCTL_RD_REG_PA_RT _IOWR(MAJO, 24, SADDR_DATA_PA) 		//DON'T USE, read from paddr RT
//!!! #define IOCTL_WR_REG_PA_DIS _IOWR(MAJO, 25, SADDR_DATA_PA_DIS)    //DON'T USE, write to paddr DIS
//!!! #define IOCTL_RD_REG_PA_DIS _IOWR(MAJO, 26, SADDR_DATA_PA_DIS)    //DON'T USE, read from paddr DIS
#define IOCTL_WR_BLOCK_BUF_PA _IOW(MAJO, 27, SBLOCK_BUF_PA) //write block
#define IOCTL_RD_BLOCK_BUF_PA _IOWR(MAJO, 28, SBLOCK_BUF_PA) //read block
//bc_ram_reg
#define IOCTL_WRITE_BC_CONTR_REG _IOW(MAJO, 29, BC_RAM_STR) // write to BC ram
#define IOCTL_READ_BC_CONTR_REG _IOWR(MAJO, 30, BC_RAM_STR) // read from BC ram
//info command
#define IOCTL_VERSION _IOR(MAJO, 31, VERSION) // getting information about board
#define IOCTL_VERSION_DRIVER _IOR(MAJO, 40, unsigned int) // getting information about driver
#define IOCTL_PCI_LOCATION _IOR(MAJO, 41, PCI_LOCATION) // getting information about pci location
//////////


// default signal
#define SIGDEFAULT SIGUSR1

#define RT_MODE			0
#define BC_MODE			1
#define BM_MODE			2
#define BM_ADDR_MODE	3

//INTERRUPT BITS
#define INTR_HDAT_1CH	 0x00000001
#define INTR_QDAT_1CH	 0x00000002
#define INTR_HDAT_2CH	 0x00000004
#define INTR_QDAT_2CH	 0x00000008
#define INTR_HDAT_3CH	 0x00000010
#define INTR_QDAT_3CH	 0x00000020
#define INTR_HDAT_4CH	 0x00000040
#define INTR_QDAT_4CH	 0x00000080
#define INTR_SADDR_1CH	 0x00000100
#define INTR_MC_ERR_1CH	 0x00000200
#define INTR_BC_1CH	 	 0x00000400
//800
#define INTR_SADDR_2CH	 0x00001000
#define INTR_MC_ERR_2CH	 0x00002000
#define INTR_BC_2CH	 	 0x00004000
//8000
#define INTR_SADDR_3CH	 0x00010000
#define INTR_MC_ERR_3CH	 0x00020000
#define INTR_BC_3CH	 	 0x00040000
//80000
#define INTR_SADDR_4CH	 0x00100000
#define INTR_MC_ERR_4CH	 0x00200000
#define INTR_BC_4CH	 	 0x00400000

//800000
#define INTR_FLASH 			0x01000000

#define SINTHIOSA_INTR_ALL	0x0000007f

//RT interrupts
#define RT_INT_MC		0x00100000UL
#define RT_INT_ERR		0x00200000UL
#define RT_INT_REN		0x00400000UL
#define RT_INT_TEN		0x00800000UL

//RT mask
#define RT_INT_TEN_MASK		0x1
#define RT_INT_REN_MASK 	0x2
#define RT_INT_ERR_MASK 	0x4
#define RT_INT_MC_MASK 		0x8

// Interrupt numbers
#define INT_HDATx		1
#define INT_QDATx		2
#define RTx_INT_MC		3
#define RTx_INT_ERR		4
#define RTx_INT_REN		5
#define RTx_INT_TEN		6
#define INT_BCx			7


#define DATA		0
#define INSTR		1
#define OPER		2

#define TRANSMIT	0		
#define RECEIVE		1

#define OFF	0		
#define ON	1

#define NONE	0
#define BUSA	1
#define BUSB	2
#define BOTH	3

#define BUF0	1
#define BUF1	2

#define DISABLE 0
#define ENABLE	1

typedef struct {
	unsigned int pBus;
	unsigned int pSlot;
} PCI_LOCATION;

typedef struct {
	unsigned int device_id;
	unsigned int vendor_id;
	unsigned int type;
	char revision;
	char dev_name[30];
	int minor;
	int irq;
	long size_dma;
	void* addr_dma_virt;
	unsigned int pci_bars;
	void* addr_bar_virt;
} VERSION;

typedef struct {
	unsigned long daddr;     //u32 0x... or reg_name (MIL_CTRL_REG_PCI)
	unsigned int data;      //u32
	//unsigned char buf1[128]; //u8
} SADDR_DATA;


typedef struct {
	unsigned int nCh;      	//Channel number
} CH_NUM;

typedef struct {
	unsigned int nCh;      	//Channel number
	unsigned int nBlock;	//DMA data blocks number
} GET_NBLOCK;

typedef struct {
	unsigned int nCh;      	//Channel number
	unsigned int nNum;	//Number of subaddress
	short 	action;		//ENABLE/DISABLE
}SADDR_INT;

typedef struct {
	unsigned int nCh;      	//Channel number
	unsigned int nNum;	//Number of subaddress
} SET_TR_MODE;

typedef struct {
	unsigned int nCh;      	//Channel number
	unsigned int nNum;		//Number of subaddress
	unsigned int nBuf;		//pick up buffer
} SET_TR_MODE_BUF;


typedef struct {
	unsigned int nCh;      	//Channel number
	unsigned int nBus;	//DMA data blocks number
} BUS_SEL;

typedef struct {
	unsigned int nCh;      	//Channel number
	short nFlag;		//DMA data blocks number
} WORK_ENABLE;

typedef struct {
	unsigned int nCh;      	//Channel number
	unsigned short nRT;		//Receive or Transmit
	/*unsigned int data; 		// data from/to paddr*/
	unsigned long paddr;	// 1-30 - for RT
} SADDR_DATA_PA;

typedef struct {
	unsigned long daddr; // MIL_DIS_RCV_REG, MIL_DIS_TR_REG
	unsigned long paddrs; // // paddress to be listening from addr (0xffffffff - ALL paddr from addr)
	unsigned long addr; // 0-31 - for BM, BM_ADDR mode (address for listening)
} SADDR_DATA_PA_DIS;

typedef struct {
	unsigned int nCh;      	//Channel number
	unsigned long num_buf;
	unsigned long paddr;
	unsigned short buf[32];
} SBLOCK_BUF_PA;

typedef struct {
	unsigned int nCh;      	//Channel number
	unsigned long type; 	//DATA, INSTR, OPER
	unsigned long offset;
	unsigned long data;
} BC_RAM_STR;

typedef struct {
	unsigned long proc_pid0;
	unsigned long sig_tosend0;
} SIG_STR;

typedef struct {
	unsigned int nCh;      	//Channel number
	short timeout_rcv; 		// 00 - 17; 01 - 60; 10 - 85; 11 - 110
	short timeout_tr; 		// 000 - 6; .....; 111 - 111
	short timer;			//1, 2, 4, 8, 16, 32, 64
} TT;

typedef struct {
	unsigned int number_block;
	unsigned int nCh;
	unsigned char *buf;
} DMA_STR;

typedef struct {
	short ad; //address 1..30
	int nCh;
} AD_STR;

typedef struct {
	int m_nInt; // INTR_HDAT, INTR_QDAT ...
	int m_nCh;
	int proc_pid;
	int sig_no;
} SINTHIOSA;

typedef struct {
	int   nCh;
	short mode; // RT_MODE, BC_MODE...
} MODE_STR;


#endif // IOCTL_MAN_H
